==> ssft-0.13.tar.gz: ssft-0.13/config.cpp <==
/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include <assert.h>
#include "ssft.h"

#ifdef WIN32
#   include <shlobj.h>
#endif

using namespace std;

const struct ConfToken token_list[] = 
{
    // Type        Len  Name        variable
    {TOKEN_STRING, 8,   "cert-dir", &cert_dir},
    {TOKEN_STRING, 8,   "log-file", &logfile_name},
    {TOKEN_STRING, 11,  "server-cert", &cert_file},
    {TOKEN_STRING, 10,  "server-key", &key_file},
    {TOKEN_STRING, 11,  "client-cert", &cli_cert_file},
    {TOKEN_STRING, 10,  "client-key", &cli_key_file},
    {TOKEN_INT,    4,   "port", &my_port},
    {TOKEN_STRING, 4,   "host", &hostname},
    {TOKEN_STRING, 8,   "pid-file", &pid_file},

    {TOKEN_INVALID, 0, NULL, NULL}
};

const ConfToken *LookupToken(const ConfToken *ptok, const char *name, size_t len)
{
    for (
        ; ptok->mType != TOKEN_INVALID
        ; ptok++)
    {
        if ((ptok->mLength == len)
            && (strnicmp(ptok->mpName, name, len) == 0))
            return ptok;
    }

    return NULL;
};

void clear_user_list()
{
    while(!user_list.empty())
    {
        delete user_list.front();
        user_list.pop_front();
    }
}

bool config_reload()
{
    if (config_file.empty())
    {
        /* Try to find a ssft.conf file somewhere...
         * If we are a server, check the global
         * locations. If we are a client, first
         * check the users home-directory
         */
    
        config_file = "ssft.conf";
        
        if (!is_file(config_file) && !server_mode)
        {
#ifdef WIN32
            string path;
            get_registry_value(HKEY_CURRENT_USER,
                "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 
                "AppData",
                path,
                NULL);

            if (!path.empty())
            {
                make_dirs(merge_paths(path, "Ssft\\ssft.conf"));
                config_file = merge_paths(path, "Ssft\\ssft.conf");
            }

            if (!is_file(config_file))
            {
                const char *p = getenv("HOME");
                if (p)
                    config_file = merge_paths(string(p), "ssft.conf");
            }
#else   
            // UNIX systems
            const char *p = getenv("HOME");
            if (p)
                config_file = merge_paths(string(p), ".ssft.conf");
#endif
        }

        if (!is_file(config_file))
        {
#ifdef WIN32
            char szPath[MAX_PATH * 2];
            memset(szPath, 0, sizeof(szPath));

            if (::GetWindowsDirectory(szPath, sizeof(szPath)))
                config_file = merge_paths(string(szPath), "ssft.conf");
            else
                config_file = "\\ssft.conf";
#else   
            // UNIX systems
            config_file = "/etc/ssft/ssft.conf";
#endif
        }
    }

    const string current_logfile_name = logfile_name;
    bool have_open_logfile = log_file.is_open();

    SectionsE current_section = SECT_INVALID;
    int line = 0;
    ifstream infile(config_file.c_str());

    if (debug_flag)
    {
        log << LOG_DATE 
            << "Loading configuration-file \""
            << config_file
            << "\""
            << ENDL;
    }
    
    if (!infile)
    {
        if (debug_flag)
        {
            log << "Failed to open configuration file \""
                << config_file
                << "\" for read."
                << ENDL;
        }
        return false;
    }
    
    // Reset
    clear_user_list();
    User *puser = NULL;

    while(infile.good())
    {
        char buf[1024 * 8];

        buf[0] = 0;
        infile.getline(buf, sizeof(buf));
        ++line;

        if (!infile.good())
            break;

        const char *p = buf;
        while(isspace(*p))
            ++p;
        if (!*p
            || ('#' == *p)
            || (';' == *p))
            continue; // Empty or comment

        if ('[' == *p)
        {
            // Start of section

            ++p;
            while(isspace(*p))
                ++p;
            if (strnicmp(p, "General", 7) == 0)
                current_section = SECT_GENERAL;
            else if (strnicmp(p, "User", 4) == 0)
            {
                if (server_mode)
                {
                    p += 4;
                    while(isspace(*p))
                        ++p;
                    if (':' == *p)
                        ++p;
                    while(isspace(*p))
                        ++p;
                    
                    puser = new User(p);
                    user_list.push_back(puser);
                }
                current_section = SECT_USER;
            }
            else
                current_section = SECT_INVALID;
        } 
        else if (isalpha(*p))
        {
            // Parameter
            const char *token_start = p;
            while(!isspace(*p) && ('=' != *p))
                ++p;
            size_t token_len = p - token_start;

            while(isspace(*p))
                ++p;
            if ('=' == *p)
                ++p;
            while(isspace(*p))
                ++p;

            // token_start is start of token
            // token_len is length of token
            // p is argument

            const ConfToken *ptok = NULL;

            switch(current_section)
            {
            case SECT_GENERAL:
                {
                    ptok = LookupToken(token_list, 
                        token_start, token_len);
have_token:
                    if (ptok)
                    {
                        switch(ptok->mType)
                        {
                        case TOKEN_STRING:
                            if (((string *)(ptok->mpPtr))->empty())
                                *(string *)(ptok->mpPtr) = p;
                            break;
                        case TOKEN_INT:
                            if (*((int *)(ptok->mpPtr)) == 0)
                                *(int *)(ptok->mpPtr) = atoi(p);
                            break;
                        default:
                            assert(false);
                        }
                    }
                    else
                    {
                        log << LOG_DATE
                            << "Warning: Invalid option \""
                            << token_start
                            << "\" in "
                            << line
                            << " of the configuration file (\""
                            << config_file
                            << "\")."
                            << ENDL;
                    }
                }
                break;
            case SECT_USER:
                if (server_mode)
                {
                    ptok = LookupToken(puser->GetTokens(), token_start, token_len);
                    goto have_token;
                }
                break;
            case SECT_INVALID:
                log << LOG_DATE
                    << "Warning: Setting outside section in line "
                    << line
                    << " of the configuration file (\""
                    << config_file
                    << "\"): \""
                    << p
                    << "\""
                    << ENDL;
            }
        }
        else
        {
            log << LOG_DATE
                << "Warning: Unrecognized token in the configuration file (\""
                << config_file
                << "\"): \""
                << p
                << "\""
                << ENDL;
        }
    }

    if (log_file.is_open())
    {
        log_file.close();
    }

    if (!logfile_name.empty())
    {
        log_file.open(logfile_name.c_str(), ios::out|ios::app);

        if (!log_file)
        {
            log << LOG_DATE 
                << "Failed to open logfile \"" 
                << logfile_name
                << "\"."
                << ENDL;
        } else if (!have_open_logfile)
        {
            log_file << "===================================================="
                << SYSENDL
                << "===================================================="
                << SYSENDL;
        }
    }

    return true;
}

==> ssft-0.16.tar.gz: ssft-0.16/config.cpp <==
/* 
    ssft - is a simple file transfer program that use SSL (Secure 
    Socket Layer) encryption to secure the transfers - and 
    signed certificates to authenticate users and servers. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Copyright 2001 - 2002 by Jarle (jgaa) Aase, jgaa@jgaa.com
    http://www.jgaa.com

*/

#include "StdAfx.h"
#include <assert.h>
#include "ssft.h"

#ifdef WIN32
//#   include <shlobj.h>
#endif

using namespace std;

const struct ConfToken token_list[] = 
{
    // Type        Len  Name        variable
    {TOKEN_STRING, 8,   "cert-dir", &cert_dir},
    {TOKEN_STRING, 8,   "log-file", &logfile_name},
    {TOKEN_STRING, 11,  "server-cert", &cert_file},
    {TOKEN_STRING, 10,  "server-key", &key_file},
    {TOKEN_STRING, 11,  "client-cert", &cli_cert_file},
    {TOKEN_STRING, 10,  "client-key", &cli_key_file},
    {TOKEN_INT,    4,   "port", &my_port},
    {TOKEN_STRING, 4,   "host", &hostname},
    {TOKEN_STRING, 8,   "pid-file", &pid_file},

    {TOKEN_INVALID, 0, NULL, NULL}
};

const ConfToken *LookupToken(const ConfToken *ptok, const char *name, size_t len)
{
    for (
        ; ptok->mType != TOKEN_INVALID
        ; ptok++)
    {
        if ((ptok->mLength == len)
            && (strnicmp(ptok->mpName, name, len) == 0))
            return ptok;
    }

    return NULL;
};

void clear_user_list()
{
    while(!user_list.empty())
    {
        delete user_list.front();
        user_list.pop_front();
    }
}

bool config_reload()
{
    if (config_file.empty())
    {
        /* Try to find a ssft.conf file somewhere...
         * If we are a server, check the global
         * locations. If we are a client, first
         * check the users home-directory
         */
    
        config_file = "ssft.conf";
        
        if (!is_file(config_file) && !server_mode)
        {
#ifdef WIN32
            string path;
            get_registry_value(HKEY_CURRENT_USER,
                "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders", 
                "AppData",
                path,
                NULL);

            if (!path.empty())
            {
                make_dirs(merge_paths(path, "Ssft\\ssft.conf"), 0, NULL, NULL);
                config_file = merge_paths(path, "Ssft\\ssft.conf");
            }

            if (!is_file(config_file))
            {
                const char *p = getenv("HOME");
                if (p)
                    config_file = merge_paths(string(p), "ssft.conf");
            }
#else   
            // UNIX systems
            const char *p = getenv("HOME");
            if (p)
                config_file = merge_paths(string(p), ".ssft.conf");
#endif
        }

        if (!is_file(config_file))
        {
#ifdef WIN32
            char szPath[MAX_PATH * 2];
            memset(szPath, 0, sizeof(szPath));

            if (::GetWindowsDirectory(szPath, sizeof(szPath)))
                config_file = merge_paths(string(szPath), "ssft.conf");
            else
                config_file = "\\ssft.conf";
#else   
            // UNIX systems
            config_file = "/etc/ssft/ssft.conf";
#endif
        }
    }

    const string current_logfile_name = logfile_name;
    bool have_open_logfile = log_file.is_open();

    SectionsE current_section = SECT_INVALID;
    int line = 0;
    ifstream infile(config_file.c_str());

    if (debug_flag)
    {
        log << LOG_DATE 
            << "Loading configuration-file \""
            << config_file
            << "\""
            << ENDL;
    }
    
    if (!infile)
    {
        if (debug_flag)
        {
            log << "Failed to open configuration file \""
                << config_file
                << "\" for read."
                << ENDL;
        }
        return false;
    }
    
    // Reset
    clear_user_list();
    User *puser = NULL;

    while(infile.good())
    {
        char buf[1024 * 8];

        buf[0] = 0;
        infile.getline(buf, sizeof(buf));
        ++line;

        if (!infile.good())
            break;

        const char *p = buf;
        while(isspace(*p))
            ++p;
        if (!*p
            || ('#' == *p)
            || (';' == *p))
            continue; // Empty or comment

        if ('[' == *p)
        {
            // Start of section

            ++p;
            while(isspace(*p))
                ++p;
            if (strnicmp(p, "General", 7) == 0)
                current_section = SECT_GENERAL;
            else if (strnicmp(p, "User", 4) == 0)
            {
                if (server_mode)
                {
                    p += 4;
                    while(isspace(*p))
                        ++p;
                    if (':' == *p)
                        ++p;
                    while(isspace(*p))
                        ++p;
                    
                    puser = new User(p);
                    user_list.push_back(puser);
                }
                current_section = SECT_USER;
            }
            else
                current_section = SECT_INVALID;
        } 
        else if (isalpha(*p))
        {
            // Parameter
            const char *token_start = p;
            while(!isspace(*p) && ('=' != *p))
                ++p;
            size_t token_len = p - token_start;

            while(isspace(*p))
                ++p;
            if ('=' == *p)
                ++p;
            while(isspace(*p))
                ++p;

            // token_start is start of token
            // token_len is length of token
            // p is argument

            const ConfToken *ptok = NULL;

            switch(current_section)
            {
            case SECT_GENERAL:
                {
                    ptok = LookupToken(token_list, 
                        token_start, token_len);
have_token:
                    if (ptok)
                    {
                        switch(ptok->mType)
                        {
                        case TOKEN_STRING:
                            if (((string *)(ptok->mpPtr))->empty())
                                *(string *)(ptok->mpPtr) = p;
                            break;
                        case TOKEN_INT:
                            if (*(int *)ptok->mpPtr == 0)
                                *(int *)(ptok->mpPtr) = atoi(p);
                            break;
                        case TOKEN_OCT:
                            if (*(int *)ptok->mpPtr == 0)
                            {
                                sscanf(p, "%o", (unsigned int *)ptok->mpPtr);
                                if (debug_flag)
                                {
                                    log << LOG_DATE << "(debug) read octal value " 
                                        << (*(int *)ptok->mpPtr) << " (des)" << ENDL;
                                }
                            }
                            break;
                        default:
                            assert(false);
                        }
                    }
                    else
                    {
                        log << LOG_DATE
                            << "Warning: Invalid option \""
                            << token_start
                            << "\" in "
                            << line
                            << " of the configuration file (\""
                            << config_file
                            << "\")."
                            << ENDL;
                    }
                }
                break;
            case SECT_USER:
                if (server_mode)
                {
                    ptok = LookupToken(puser->GetTokens(), token_start, token_len);
                    goto have_token;
                }
                break;
            case SECT_INVALID:
                log << LOG_DATE
                    << "Warning: Setting outside section in line "
                    << line
                    << " of the configuration file (\""
                    << config_file
                    << "\"): \""
                    << p
                    << "\""
                    << ENDL;
            }
        }
        else
        {
            log << LOG_DATE
                << "Warning: Unrecognized token in the configuration file (\""
                << config_file
                << "\"): \""
                << p
                << "\""
                << ENDL;
        }
    }

    if (log_file.is_open())
    {
        log_file.close();
    }

    if (!logfile_name.empty())
    {
        log_file.open(logfile_name.c_str(), ios::out|ios::app);

        if (!log_file)
        {
            log << LOG_DATE 
                << "Failed to open logfile \"" 
                << logfile_name
                << "\"."
                << ENDL;
        } else if (!have_open_logfile)
        {
            log_file << "===================================================="
                << SYSENDL
                << "===================================================="
                << SYSENDL;
        }
    }

    return true;
}
